use crate::commands::command_base::{
    BasicWritableCommand, DeserializableCommand, DeserializedCommand, SerializableCommand,
};

#[derive(new)]
pub struct ProgramInput {
    pub mix_effect: u8,
    pub source: u16,
}

impl SerializableCommand for ProgramInput {
    fn payload(&self, _version: crate::enums::ProtocolVersion) -> Vec<u8> {
        let mut buf = vec![0; 4];
        buf[..1].copy_from_slice(&self.mix_effect.to_be_bytes());
        buf[2..].copy_from_slice(&self.source.to_be_bytes());

        buf
    }
}

impl BasicWritableCommand for ProgramInput {
    fn get_raw_name(&self) -> &'static str {
        "CPgI"
    }

    fn get_minimum_version(&self) -> crate::enums::ProtocolVersion {
        crate::enums::ProtocolVersion::Unknown
    }
}

impl DeserializableCommand for ProgramInput {
    fn get_raw_name(&self) -> &'static str {
        "PrgI"
    }

    fn get_minimum_version(&self) -> crate::enums::ProtocolVersion {
        crate::enums::ProtocolVersion::Unknown
    }
}

impl DeserializedCommand for ProgramInput {
    fn deserialize(buffer: &[u8]) -> Self
    where
        Self: Sized,
    {
        let mix_effect = buffer[0];
        let source = u16::from_be_bytes([buffer[2], buffer[3]]);

        Self { mix_effect, source }
    }

    fn apply_to_state(&self, state: &mut crate::state::AtemState) -> Vec<String> {
        todo!()
    }
}
